# Generated by Django 2.2.14 on 2020-07-17 08:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_apscheduler", "0003_auto_20200716_1632"),
    ]

    operations = [
        migrations.AlterField(
            model_name="djangojob",
            name="id",
            field=models.CharField(
                help_text="Unique id for this job.",
                max_length=255,
                primary_key=True,
                serialize=False,
            ),
        ),
        migrations.AlterField(
            model_name="djangojobexecution",
            name="status",
            field=models.CharField(
                choices=[
                    ("Started execution", "Started execution"),
                    ("Error!", "Error!"),
                    ("Executed", "Executed"),
                ],
                help_text="The current status of this job execution.",
                max_length=50,
            ),
        ),
    ]
