# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-12 07:58
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_apscheduler", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="djangojob",
            name="next_run_time",
            field=models.DateTimeField(blank=True, db_index=True, null=True),
        ),
    ]
