# Generated by Django 2.2.14 on 2020-07-17 09:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_apscheduler", "0005_migrate_name_to_id"),
    ]

    operations = [
        migrations.RemoveField(model_name="djangojob", name="name",),
    ]
